#!/usr/bin/perl -w
#
#  Function to process command line parameters
#
#  Copyright (c) 2001  Willem A. Schreuder <willem@prinmath.com>
#  This file is part of the KUDU suite of software.
#
#  The KUDU programs are free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  The KUDU programs are distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along 
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
#  See also www.fsf.org

use strict;

sub GetOpt
{
   my ($args,%def) = @_;
   ($args =~ /-/) && die "Unexpected - in $args\n";
   #  First group are singles, each subsequent is a long options
   my %opt;
   my ($single,@args) = split(' ',$args);
   #  Long options
   foreach my $long (@args)
   {
      if ($long =~ s/(:+)$//)
      {
         $opt{$long} = length $1;
      }
      else
      {
         $opt{$long} = 0;
      }
   }
   #  Single options
   @args = split(/ */,$single);
   while (@args>0)
   {
      my $arg = shift @args;
      ($arg eq ':') && die "Unexpected : in $args\n";
      $opt{$arg} = 0;
      while (@args>0 && $args[0] eq ':')
      {
         shift @args;
         $opt{$arg}++;
      }
   }
   #  Process command line
   my %args;
   while (@ARGV>0 && $ARGV[0] =~ /^-./)
   {
      my $arg = shift @ARGV;
      my ($key,$val);
      #  --foo=bar
      if ($arg =~ /^--(.+)=(.*)$/)
      {
         ($key,$val) = ($1,$2);
         exists($opt{$key}) || die "Unknown option $arg\n";
         ($opt{$key}>0 && $val eq '') && die "Missing argument for $arg";
      }
      #  --foo
      elsif ($arg =~ /^--(.+)$/)
      {
         $key = $1;
         exists($opt{$key}) || die "Unknown option $arg\n";
         if ($opt{$key}>0)
         {
            (@ARGV>0) || die "Missing argument for $arg";
            $val = shift @ARGV;
         }
      }
      #  -foo
      elsif ($arg =~ /^-(.)(.*)$/)
      {
         ($key,$val) = ($1,$2);
         exists($opt{$key}) || die "Unknown option $key in $arg\n";
         if ($opt{$key}>0 && $val eq '')
         {
            (@ARGV==0) && die "Missing argument for $arg";
            $val = shift @ARGV;
         }
         ($opt{$key}==0 && $val) && (unshift @ARGV , "-$val");
      }
      #  WTF
      else
      {
         die "WTF: $arg\n";
      }
      if ($opt{$key}==0)
      {
         $args{$key}++;
      }
      elsif ($opt{$key}==1)
      {
         $args{$key} = $val;
      }
      else
      {
         push @{$args{$key}} , $val;
      }
   }
   while (my ($key,$val) = each(%def))
   {
      exists($args{$key}) || ($args{$key} = $val);
   }
   return %args;
}

#
#  Add optional parameters from the file
#  Does not override command line
#
sub FileOpt
{
   my ($args,$cargs,$line) = @_;
   if ($line =~ /^-(.) *(.*)/)
   {
      my ($arg,$val) = ($1,$2);
      my @args = split(/ */,$args);
      my $pos = index($args,$arg);
      ($pos >=0) || die "Unknown option: $arg\n";
      if ($pos<$#args && $args[$pos+1] eq ':')
      {
         ($val ne "") || die "Missing argument for -$arg";
      }
      else
      {
         $cargs->{$arg} = 1;
      }
      $cargs->{$arg} = $val unless (exists($cargs->{$arg}));
      return 1;
   }
   else
   {
      return 0;
   }
}

#
#  Set defaults for arguments not yet set
#
sub DefOpt
{
   my ($args,%def) = @_;
   while (my ($key,$val) = each(%def))
   {
      exists($args->{$key}) || ($args->{$key} = $val);
   }
}

1;
